<?php

declare(strict_types=1);

namespace Drupal\graphql_compose_preview\KeyValueStore;

use Drupal\Component\Utility\Crypt;
use Drupal\Core\KeyValueStore\DatabaseStorageExpirable;

/**
 * Utility methods to interact with the tokened private temp store.
 */
class DatabaseStorageExpirableToken extends DatabaseStorageExpirable {

  /**
   * {@inheritdoc}
   *
   * Add preview_token.
   */
  protected function doSetWithExpire($key, $value, $expire) {
    $value->preview_token = Crypt::randomBytesBase64(64);

    return parent::doSetWithExpire($key, $value, $expire);
  }

  /**
   * Load by token.
   *
   * @param ?string $token
   *   The token to load from the private temp store.
   *
   * @return string|bool
   *   The value.
   */
  public function getKeyByToken(?string $token) {
    $token_length = strlen($token ?? '');
    $token_pattern = sprintf('%%s:%s:"%s";%%', $token_length, $token);

    try {
      return $this->connection->query(
        'SELECT [name] FROM {' . $this->connection->escapeTable($this->table) . '} WHERE value LIKE :token',
        [
          ':token' => $token_pattern,
        ])->fetchField();
    }
    catch (\Exception $e) {
      $this->catchException($e);
    }
    return FALSE;
  }

  /**
   * Load by token.
   *
   * @param string $name
   *   The name to load from the private temp store.
   *
   * @return string|bool
   *   The value.
   */
  public function getTokenByKey($name) {
    try {
      $preview_blob = $this->connection->query(
        'SELECT [value] FROM {' . $this->connection->escapeTable($this->table) . '} WHERE name = :name',
        [
          ':name' => $name,
        ])->fetchField();

      $preview_object = $this->serializer->decode($preview_blob);

      return $preview_object?->preview_token ?? NULL;
    }
    catch (\Exception $e) {
      $this->catchException($e);
    }
    return FALSE;
  }

}
